// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:dog/dog.dart';
import 'package:ironsource_mediation/ironsource_mediation.dart';
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:adpluginx/AdBase/AppLovin/AppLovinReward.dart';
import 'package:adpluginx/Engine/Model/ScreenWiseModel.dart';
import 'package:adpluginx/adpluginx.dart';
import '../../AdBase/AppLovin/FullScreen/AppLovinFullScreenX.dart';

class FullScreenEngine {
  static final FullScreenEngine _singleton = FullScreenEngine._internal();

  factory FullScreenEngine() {
    return _singleton;
  }

  FullScreenEngine._internal();

  Map routeIndex = {};
  int currentAdIndex = 0;
  int failCounter = 0;
  int MAX_RETRY = 3;

  ScreenWiseModel? screenWiseModel;

  Function()? onJobComplete;

  indexIncrement({String? route, bool isDefault = false}) {
    dog.i(" Incremetn =====> $route");
    if (screenWiseModel != null) {
      int arrayLength = screenWiseModel!.localClick!.length - 1;
      if (route != null) {
        dog.i("Route Not Null ======> $route");
        if (routeIndex[route] == null) {
          dog.i("Route INit ======>");
          if ((arrayLength + 1) == 1) {
            routeIndex[route] = 0;
          } else {
            routeIndex[route] = 1;
          }
        } else {
          if (routeIndex[route] != arrayLength &&
              routeIndex[route] < arrayLength) {
            dog.i("Route Increment ======>");
            routeIndex[route]++;
          } else {
            dog.i("Route Reset ======>");
            routeIndex[route] = 0;
          }
        }
      } else {
        if (currentAdIndex < arrayLength && currentAdIndex != arrayLength) {
          currentAdIndex++;
        } else {
          currentAdIndex = 0;
        }
      }
    }
    if (onJobComplete != null) {
      onJobComplete!();
    }
  }

  bool loopBreaker() {
    if (failCounter < MAX_RETRY) {
      failCounter++;
      return false;
    }
    failCounter = 1;
    return true;
  }

  showAds({required BuildContext context, required Function() onJobComplete}) {
    this.onJobComplete = onJobComplete;

    ModalRoute? route = ModalRoute.of(context);
    AdBase adBase = context.read<AdBase>();
    Map? local = adBase.data![PackageInfoX().version ?? "0.0.1"]['screens']
        [route?.settings.name];
    dog.i("Route ======>${route?.settings.name}");
    if (local != null) {
      // Have Route Locals
      dog.i("Have Rote Data ==> ${routeIndex[route?.settings.name]}");
      screenWiseModel = ScreenWiseModel(
          banner: local['banner'] ?? 0,
          native: local['native'] ?? 0,
          localAdFlag: local['localAdFlag'] ?? false,
          localClick: local['localClick'] ?? [0, 0, 0],
          localFail: local['localFail'] ?? {"0": 1, "1": 2, "2": 3, "3": 0});
    } else {
      dog.i("Global Data");

      // Take it Global
      screenWiseModel = ScreenWiseModel(
        banner: adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalBanner'] ??
            0,
        native: adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalNative'] ??
            0,
        localAdFlag:
            adBase.data![PackageInfoX().version ?? "0.0.1"]['isAdsOn'] ?? false,
        localClick: (adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalClick']) ??
            [0, 0, 0],
        localFail: adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalFail'] ??
            {"0": 1, "1": 2, "2": 3, "3": 0},
      );
    }

    _showAd(route: route?.settings.name, context: context);
  }

  showActionAds(
      {required BuildContext context,
      required String actionName,
      required Function() onJobComplete}) {
    this.onJobComplete = onJobComplete;
    AdBase adBase = context.read<AdBase>();
    Map? local =
        adBase.data![PackageInfoX().version ?? "0.0.1"]['actions'][actionName];
    if (local != null) {
      // Have Route Locals
      dog.i("Local Founded");
      screenWiseModel = ScreenWiseModel(
          banner: local['banner'] ?? 0,
          native: local['native'] ?? 0,
          localAdFlag: local['localAdFlag'] ?? false,
          localClick: local['localClick'] ?? [0, 0, 0],
          localFail: local['localFail'] ?? {"0": 1, "1": 2, "2": 3, "3": 0});
    } else {
      dog.i("Global Data");

      // Take it Global
      screenWiseModel = ScreenWiseModel(
        banner: adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalBanner'] ??
            0,
        native: adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalNative'] ??
            0,
        localAdFlag:
            adBase.data![PackageInfoX().version ?? "0.0.1"]['isAdsOn'] ?? false,
        localClick: (adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalClick']) ??
            [0, 0, 0],
        localFail: adBase.data![PackageInfoX().version ?? "0.0.1"]
                ['globalFail'] ??
            {"0": 1, "1": 2, "2": 3, "3": 0},
      );
    }
    _showAd(route: actionName, context: context);
  }

  _showAd({String? route, required BuildContext context}) {
    AdBase adBase = context.read<AdBase>();
    int index = route != null ? (routeIndex[route] ?? 0) : currentAdIndex;
    if ((adBase.data![PackageInfoX().version ?? "0.0.1"]['isAdsOn'] ?? false) ==
        false) {
      if (onJobComplete != null) {
        onJobComplete!();
      }
      return;
    }
    if ((screenWiseModel!.localAdFlag ?? false) == false) {
      if (onJobComplete != null) {
        onJobComplete!();
      }
      return;
    }
    switch (screenWiseModel!.localClick![index]) {
      case 0:
        dog.i("Case Google");
        GoogleCallerX().loadGoogleAd(
          adType: GoogleAdType.fullScreen,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(0, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 1:
        dog.i("Case Facebook");
        FacebookCallerX().loadFacebook(
          adType: FacebookAdType.fullScreen,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(1, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 2:
        dog.i("Case Unity");

        UnityCallerX().loadUnityAd(
          adType: UnityAdType.fullScreen,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(2, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;

      case 3:
        IronSourceFullScreenX ironSourceFullScreenX =
            IronSourceFullScreenX.onInit(
          onClose: () {
            indexIncrement(route: route);
          },
          onFailed: () {
            dog.e("----- Failed on IronSource ------------");
            _failed(3, route: route);
          },
          onLoaded: () {
            IronSource.showInterstitial();
          },
        );
        IronSource.setISListener(ironSourceFullScreenX);
        IronSource.loadInterstitial();
        break;
      case 4:
        dog.i("Case Google Reward Normal");
        GoogleCallerX().loadGoogleAd(
          adType: GoogleAdType.rewardNormal,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(4, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 5:
        dog.i("Case Google Reward Video");
        GoogleCallerX().loadGoogleAd(
          adType: GoogleAdType.rewardVideo,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(5, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 6:
        dog.i("Case Unity Reward Video");
        UnityCallerX().loadUnityAd(
          adType: UnityAdType.rewardVideo,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(6, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 7:
        dog.i("Case Unity Normal Full Video");
        UnityCallerX().loadUnityAd(
          adType: UnityAdType.normalFullScreen,
          callerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(7, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 8:
        dog.i("Case AppLovin Normal Full Video");
        AppLovinFullScreenX().callAds(
          adCallerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(8, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      case 9:
        dog.i("Case AppLovin Reward Full Video");
        AppLovinReward().callAds(
          adCallerInterface: AdCallerInterface(
            onClose: () {
              indexIncrement(route: route);
            },
            onError: () {
              _failed(9, route: route);
            },
            onStarted: () {},
            onLoaded: () {},
            onFailedToLoad: () {},
          ),
        );
        break;
      default:
        indexIncrement(route: route, isDefault: true);
    }
  }

  _failed(int from, {String? route}) {
    dog.i("Failed From =====> $from");
    Map? failedMapArray = screenWiseModel!.localFail;
    int caseIndex = failedMapArray![from.toString()];
    dog.i("Before Loop breaker======>");

    if (loopBreaker()) {
      dog.i("Loop Break======>");
      if (onJobComplete != null) {
        onJobComplete!();
      }
    } else {
      dog.i("After Loop Breaker======>");
      switch (caseIndex) {
        case 0:
          dog.i("Case Google Failed");
          GoogleCallerX().loadGoogleAd(
            adType: GoogleAdType.fullScreen,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(caseIndex, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 1:
          dog.i("Case Facebook Failed");
          FacebookCallerX().loadFacebook(
            adType: FacebookAdType.fullScreen,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(caseIndex, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 2:
          dog.i("Case Unity Failed");
          UnityCallerX().loadUnityAd(
            adType: UnityAdType.fullScreen,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(caseIndex, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 3:
          dog.i("Case IronSource Failed");
          if (onJobComplete != null) {
            onJobComplete!();
          }
          break;
        case 4:
          dog.i("Case Google Reward Normal");
          GoogleCallerX().loadGoogleAd(
            adType: GoogleAdType.rewardNormal,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(4, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 5:
          dog.i("Case Google Reward Video");
          GoogleCallerX().loadGoogleAd(
            adType: GoogleAdType.rewardVideo,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(5, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 6:
          dog.i("Case Unity Reward Video");
          UnityCallerX().loadUnityAd(
            adType: UnityAdType.rewardVideo,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(6, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 7:
          dog.i("Case Unity Normal Full Video");
          UnityCallerX().loadUnityAd(
            adType: UnityAdType.normalFullScreen,
            callerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(7, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 8:
          dog.i("Case AppLovin Normal Full Video");
          AppLovinFullScreenX().callAds(
            adCallerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(8, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        case 9:
          dog.i("Case AppLovin Reward Full Video");
          AppLovinReward().callAds(
            adCallerInterface: AdCallerInterface(
              onClose: () {
                indexIncrement(route: route);
              },
              onError: () {
                _failed(9, route: route);
              },
              onStarted: () {},
              onLoaded: () {},
              onFailedToLoad: () {},
            ),
          );
          break;
        default:
          indexIncrement(route: route, isDefault: true);
      }
    }
  }
}
